# Autogenerated by nbdev

d = { 'settings': { 'branch': 'main',
                'doc_baseurl': '/jaxpole',
                'doc_host': 'https://rodrigodzf.github.io',
                'git_url': 'https://github.com/rodrigodzf/jaxpole',
                'lib_path': 'jaxpole'},
  'syms': { 'jaxpole.filter': { 'jaxpole.filter.all_pole_direct_form_I': ('filter.html#all_pole_direct_form_i', 'jaxpole/filter.py'),
                                'jaxpole.filter.allpole': ('filter.html#allpole', 'jaxpole/filter.py'),
                                'jaxpole.filter.allpole_bwd': ('filter.html#allpole_bwd', 'jaxpole/filter.py'),
                                'jaxpole.filter.allpole_fwd': ('filter.html#allpole_fwd', 'jaxpole/filter.py')},
            'jaxpole.test_grad': { 'jaxpole.test_grad.create_test_inputs': ('test_grad.html#create_test_inputs', 'jaxpole/test_grad.py'),
                                   'jaxpole.test_grad.get_random_biquads': ('test_grad.html#get_random_biquads', 'jaxpole/test_grad.py'),
                                   'jaxpole.test_grad.test_low_order': ('test_grad.html#test_low_order', 'jaxpole/test_grad.py')}}}
